/**
 * @file 
 * API c++ for DPE (Data Processing Engine). 
 * Copyright (C) 2022 Lukas Marek
 * @author  Lukas Marek <lukas.marek@advacam.cz>
 */

#ifndef DPEAPI_H
#define DPEAPI_H

#include <stdio.h> 
#include <stdlib.h> 
#include <vector>
#include <string>

// Error Codes:
#define DPE_ERR_NO_ERR              0           ///< No error occurred.
#define DPE_ERR_NO_DATA_POSTPROC    -10         ///< No data for data post processing.

// Cluster variables names
#define CLVAR_DETECTOR_ID    "DetectorID"        ///< Name of detector id.     
#define CLVAR_EVENT_ID       "EventID"           ///< Name of event id (coincidence group).
#define CLVAR_X              "x"                 ///< Name of x coordinated.
#define CLVAR_Y              "y"                 ///< Name of y coordinated.
#define CLVAR_TIME           "t"                 ///< Name of time.
#define CLVAR_FLAGS          "Flags"             ///<
#define CLVAR_SIZE           "Size"              ///<
#define CLVAR_ENERGY         "E"                 ///<
#define CLVAR_HEIGHT         "Height"            ///<
#define CLVAR_BORDER_PIX_N   "BorderPixCount"    ///<
#define CLVAR_ROUND          "Roundness"         ///<
#define CLVAR_LIN            "Linearity"         ///<
#define CLVAR_ANGLE          "Angle"             ///<
#define CLVAR_LENGHT         "Length"            ///<  
#define CLVAR_WIDTH          "Width"             ///<
#define CLVAR_THIN           "Thin"              ///<
#define CLVAR_THICK          "Thick"             ///<
#define CLVAR_CURLYTHIN      "CurlyThin"         ///<
#define CLVAR_LET            "LET"               ///<
#define CLVAR_WSTD_ALONG     "WStdAlong"         ///<
#define CLVAR_WSTD_PERP      "WStdPerp"          ///<
#define CLVAR_EPIX_MEAN      "EpixMean"          ///<

/// Construct DPE at the beginning of processing.
void DPE_Construct();

/// Destruc DPE at the end of processing.
void DPE_Destruct();

/// Load file with parameters of DPE.
/// @param FileParam_Path full path to file with parameters.
/// @param FileParam_Name name of file with parameters.
/// @return 0 if OK, otherwise error code.
int DPE_LoadParamFile(std::string FileParam_Path="./", std::string FileParam_Name="ParamFile.txt");

/// Set order of cluster variable names as they will be given in the cluster add.
/// It has to be used before init as part of DPE general setting or load of param. file.
/// @param  ClusterVarNames names of cluster variables in intended order for cluster add
///         function. Names of cluster variables should be taken from macros (CLVAR_...)
///         Example: ClusterVarNames = {CLVAR_DETECTOR_ID, CLVAR_EVENT_ID, CLVAR_SIZE}.
///         If not used before init or empty vector given, default order is used as listed:
///         CLVAR_DETECTOR_ID, CLVAR_EVENT_ID, CLVAR_X, CLVAR_Y, CLVAR_TIME, CLVAR_FLAGS,
///         CLVAR_SIZE, CLVAR_ENERGY, CLVAR_HEIGHT, CLVAR_BORDER_PIX_N, CLVAR_ROUND, CLVAR_LIN,
///         CLVAR_ANGLE, CLVAR_LENGHT, CLVAR_WIDTH.
/// @return 0 if OK, otherwise error code.
int DPE_SetClusterVarOrder(std::vector<std::string> ClusterVarNames = {});

/// Init DPE after setting its parameters. It is mainly for modules initialization
/// and propagation of parameters into them. Export directories are also created (if needed).
/// @return 0 if OK, otherwise error code.
int DPE_Init();

/// Reset DPE data storage and physical products (includes mainly reset of modules).
/// Settings remains the same as set with the parameters file or individual sets.
/// @return 0 if OK, otherwise error code.
int DPE_Reset();

/// Reset DPE current data storage (sets them to 0). The total and other storage are not influenced.
/// If the storage should be reset between cluster processing then this function should be called 
/// between them (DPE_ProcessClusters(); DPE_ResetCurrDataStorage(); ;DPE_ProcessClusters();)  
/// @return 0 if OK, otherwise error code.
int DPE_ResetCurrDataStorage();

/// Process given clustersDPE_AddClusters. This includes also calculation of 
/// physical products as dose, flux etc. Clusters are after processing deleted from DPE to avoid
/// several processing of the same data.
/// @param Clusters vector of vectors with cluster variables which should be processed. Inner
///        vectors include variables of clusters in order given by function DPE_SetClusterVarOrder.
/// @see DPE_SetClusterVarOrder
/// @return 0 if OK, otherwise error code.
int DPE_ProcessClusters(std::vector<std::vector<double> > Clusters);

/// Process data file. If input parameters are not set then the function uses
/// values given in the parameters file (if loaded).
/// @param FileIn_Path full path to the data file.
/// @param FileIn_Name name of the data file.d
/// @param FileIn_NameEnd name end (e.g. extension ".txt") of the data file. 
/// @return 0 if OK, otherwise error code.
int DPE_ProcessFile(std::string FileIn_Path = "", std::string FileIn_Name = "", std::string FileIn_NameEnd = "");

/// Recognition of radiation field with distance comparator and significant vectors. 
/// @param Name of the recognized radiation field (e.g. Cs137).
/// @param Prob probability of recognition success.
/// @return 0 if OK, otherwise error code.
int DPE_RadFieldRecognition(std::string& Name, double& Prob );

/// Retrieves test projection results from Compton camera directional analysis.
/// @param  Projection is an image defined in config file of Compton cam.
///         It includes back projection of Compton cones.
/// @return 0 if OK, otherwise error code.
int DPE_GetComCamTestProjection(std::vector<double>& Projection);

/// Retrieves projection results from Compton camera directional analysis.
/// @param  Projection is an image defined in config file of Compton cam.
///         It includes back projection of Compton cones.
/// @return 0 if OK, otherwise error code.
int DPE_GetComCamProjection(std::vector<double>& Projection);



/// Gets current deposited energy.
/// @param  EnergyDep sum of deposited energy is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentEnergyDep(double& EnergyDep);

/// Gets current dose.
/// @param  Dose sum of dose is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentDose(double& Dose);

/// Gets current dose rate.
/// @param  DoseRate mean of dose rate is stored there.
/// @param  Time used for calculation.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentDoseRate(double& DoseRate, double Time = 0);

/// Gets current count of particles.
/// @param  CountParticle sum of count of particles is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentCountParticle(double& CountParticle);

/// Gets current count of rate of particles for time sample.
/// @param  CountRate mean of count rate is stored there.
/// @param  Time used for calculation.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentCountRate(double& CountRate, double Time = 0);

/// Gets current fluence.
/// @param  Fluence sum of fluence is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentFluence(double& Fluence);

/// Gets current flux.
/// @param  Flux sum of flux is stored there.
/// @param  Time used for calculation.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentFlux(double& Flux, double Time = 0);


/// Gets current deposited energy for individual particle classes.
/// @param  EnergyDepClass sum of deposited energy for individual particle classes is 
/// stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentEnergyDepClass(std::vector<double>& EnergyDepClass);

/// Gets current dose for individual particle classes.
/// @param  DoseClass sum of dose for individual particle classes is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentDoseClass(std::vector<double>& DoseClass);

/// Gets current dose rate for individual particle classes.
/// @param  DoseRateClass mean of dose rate for individual particle classes is stored 
/// there.
/// @param  Time used for calculation.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentDoseRateClass(std::vector<double>& DoseRateClass, double Time = 0);

/// Gets current count of particles for individual particle classes.
/// @param  CountParticleClass sum of count of particles for individual particle classes 
/// is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentCountParticleClass(std::vector<double>& CountParticleClass);

/// Gets current count of rate of particles for time sample.
/// @param  CountRateClass mean of count rate for individual particle classes is stored 
/// there.
/// @param  Time used for calculation for individual particle classes.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentCountRateClass(std::vector<double>& CountRateClass, double Time = 0);

/// Gets current fluence for individual particle classes.
/// @param  FluenceClass sum of fluence for individual particle classes is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentFluenceClass(std::vector<double>& FluenceClass);

/// Gets current flux for individual particle classes.
/// @param  FluxClass sum of flux for individual particle classes is stored there.
/// @param  Time used for calculation.
/// @return 0 if OK, otherwise error code.
int DPE_GetCurrentFluxClass(std::vector<double>& FluxClass, double Time = 0);


/// Gets deposited energy.
/// @param  EnergyDep sum of deposited energy is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetEnergyDep(double& EnergyDep);

/// Gets dose.
/// @param  Dose sum of dose is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetDose(double& Dose);

/// Gets dose rate.
/// @param  DoseRate mean of dose rate is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetDoseRate(double& DoseRate);

/// Gets count of particles.
/// @param  CountParticle sum of count of particles is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCountParticle(double& CountParticle);

/// Gets count of rate of particles for time sample.
/// @param  CountRate mean of count rate is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetCountRate(double& CountRate);

/// Gets fluence.
/// @param  Fluence sum of fluence is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetFluence(double& Fluence);

/// Gets flux.
/// @param  Flux vector sum of flux is stored there.
/// @return 0 if OK, otherwise error code.
int DPE_GetFlux(double& Flux);


/// Gets deposited energy for time sample.
/// @param  EnergyDepTime vector is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1, TimeSample2,...}
/// @return 0 if OK, otherwise error code.
int DPE_GetEnergyDepTime(std::vector<double>& EnergyDepTime);

/// Gets dose for time sample.
/// @param  DoseTime vector is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1, TimeSample2,...}
/// @return 0 if OK, otherwise error code.
int DPE_GetDoseTime(std::vector<double>& DoseTime);

/// Gets dose rate for time sample.
/// @param  DoseRateTime vector is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1, TimeSample2,...}
/// @return 0 if OK, otherwise error code.
int DPE_GetDoseRateTime(std::vector<double>& DoseRateTime);

/// Gets count of particles for time sample.
/// @param  CountParticleTime vector is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1, TimeSample2,...}
/// @return 0 if OK, otherwise error code.
int DPE_GetCountParticleTime(std::vector<double>& CountParticleTime);

/// Gets count of rate of particles for time sample.
/// @param  CountRateTime vector is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1, TimeSample2,...}
/// @return 0 if OK, otherwise error code.
int DPE_GetCountRateTime(std::vector<double>& CountRateTime);

/// Gets fluence of particles for time sample.
/// @param  FluenceTime vector is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1, TimeSample2,...}
/// @return 0 if OK, otherwise error code.
int DPE_GetFluenceTime(std::vector<double>& FluenceTime);

/// Gets flux for time sample.
/// @param  FluxTime vector is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1, TimeSample2,...}
/// @return 0 if OK, otherwise error code.
int DPE_GetFluxTime(std::vector<double>& FluxTime);


/// Gets deposited energy for time sample and class.
/// @param  EnergyDepTimeClass inner vector is over particle classes and the outer
///         is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1={Class1, Class2, ...}, TimeSample2={Class1, Class2, ...},...}
/// @return 0 if OK, otherwise error code.
int DPE_GetEnergyDepTimeClass(std::vector<std::vector<double> >& EnergyDepTimeClass);

/// Gets dose for time sample and class.
/// @param  DoseTimeClass inner vector is over particle classes and the outer
///         is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1={Class1, Class2, ...}, TimeSample2={Class1, Class2, ...},...}
/// @return 0 if OK, otherwise error code.
int DPE_GetDoseTimeClass(std::vector<std::vector<double> >& DoseTimeClass);

/// Gets dose rate for time sample and class.
/// @param  DoseRateTimeClass inner vector is over particle classes and the outer
///         is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1={Class1, Class2, ...}, TimeSample2={Class1, Class2, ...},...}
/// @return 0 if OK, otherwise error code.
int DPE_GetDoseRateTimeClass(std::vector<std::vector<double> >& DoseRateTimeClass);

/// Gets count of particles for time sample and class.
/// @param  CountParticleTimeClass inner vector is over particle classes and the outer
///         is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1={Class1, Class2, ...}, TimeSample2={Class1, Class2, ...},...}
/// @return 0 if OK, otherwise error code.
int DPE_GetCountParticleTimeClass(std::vector<std::vector<double> >& CountParticleTimeClass);

/// Gets count of rate of particles for time sample and class.
/// @param  CountRateTimeClass inner vector is over particle classes and the outer
///         is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1={Class1, Class2, ...}, TimeSample2={Class1, Class2, ...},...}
/// @return 0 if OK, otherwise error code.
int DPE_GetCountRateTimeClass(std::vector<std::vector<double> >& CountRateTimeClass);

/// Gets fluence of particles for time sample and class.
/// @param  FluenceTimeClass inner vector is over particle classes and the outer
///         is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1={Class1, Class2, ...}, TimeSample2={Class1, Class2, ...},...}
/// @return 0 if OK, otherwise error code.
int DPE_GetFluenceTimeClass(std::vector<std::vector<double> >& FluenceTimeClass);

/// Gets flux for time sample and class.
/// @param  FluxTimeClass inner vector is over particle classes and the outer
///         is over time samples (based on time sapling and elapsed time).
///         Example: {TimeSample1={Class1, Class2, ...}, TimeSample2={Class1, Class2, ...},...}
/// @return 0 if OK, otherwise error code.
int DPE_GetFluxTimeClass(std::vector<std::vector<double> >& FluxTimeClass);

/// Gets Compton camera count of bins on X and Y axis for its projection.
/// @param  ComCamProjNBinX is count of bins on X axis.
/// @param  ComCamProjNBinY is count of bins on Y axis.
/// @return 0 if OK, otherwise error code.
int DPE_GetComCamProjectionNBins(int& ComCamProjNBinX, int& ComCamProjNBinY);

/// Gets Compton camera maximum value in its projection.
/// @param  ComCamProjNBinX maximum value.
/// @return 0 if OK, otherwise error code.
int DPE_GetComCamProjectionMaxVal(double& MaxVal);


/// Sets time sampling.
/// @param  TimeSampling sampling time which defines a sample evaluation.
///         Its unit is in seconds. Defaults value is 1 second.
/// @return 0 if OK, otherwise error code.
int DPE_SetTimeSampling(double TimeSampling);

/// Sets time acquisition.
/// @param  TimeAcquisition acquisition time which is used for calculation
///         of physical products. Default is set during processing based on 
///         given data.
/// @return 0 if OK, otherwise error code.
int DPE_SetTimeAcquisition(double TimeAcquisition);

/// Sets Compton camera distance for projection.
/// @param  ProjectionDist in which projection is calculated. It is in mm.
/// @return 0 if OK, otherwise error code.
int DPE_SetComCamProjectionDist(double ProjectionDist);


#endif /* !DPEAPI_H */
